import { createRouteRef, useApi, configApiRef, errorApiRef, createPlugin, createComponentExtension } from '@backstage/core-plugin-api';
import React, { createContext, useContext, useState, useEffect, Component, useMemo, forwardRef } from 'react';
import { union, literal, object, string, optional, array, number, parse, ValiError, undefined_, record } from 'valibot';
import { scmIntegrationsApiRef } from '@backstage/integration-react';
import { useEntity, getEntitySourceLocation } from '@backstage/plugin-catalog-react';
import { useQuery, useQueryClient, QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { makeStyles } from '@material-ui/core';
import { LinkButton, Link, InfoCard } from '@backstage/core-components';
import TextField from '@material-ui/core/TextField';
import ErrorIcon from '@material-ui/icons/ErrorOutline';
import SyncIcon from '@material-ui/icons/Sync';

const rootRouteRef = createRouteRef({
  id: "coder"
});

const workspaceAgentStatusSchema = union([
  literal("connected"),
  literal("connecting"),
  literal("disconnected"),
  literal("timeout")
]);
const workspaceAgentSchema = object({
  id: string(),
  status: workspaceAgentStatusSchema
});
const workspaceResourceSchema = object({
  id: string(),
  agents: optional(array(workspaceAgentSchema))
});
const workspaceStatusSchema = union([
  literal("canceled"),
  literal("canceling"),
  literal("deleted"),
  literal("deleting"),
  literal("failed"),
  literal("pending"),
  literal("running"),
  literal("starting"),
  literal("stopped"),
  literal("stopping")
]);
const workspaceBuildSchema = object({
  id: string(),
  resources: array(workspaceResourceSchema),
  status: workspaceStatusSchema
});
const workspaceSchema = object({
  id: string(),
  name: string(),
  template_icon: string(),
  owner_name: string(),
  latest_build: workspaceBuildSchema
});
const workspaceBuildParameterSchema = object({
  name: string(),
  value: string()
});
const workspaceBuildParametersSchema = array(
  workspaceBuildParameterSchema
);
const workspacesResponseSchema = object({
  count: number(),
  workspaces: array(workspaceSchema)
});

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _response;
const CODER_QUERY_KEY_PREFIX = "coder-backstage-plugin";
const PROXY_ROUTE_PREFIX = "/api/proxy/coder";
const API_ROUTE_PREFIX = `${PROXY_ROUTE_PREFIX}/api/v2`;
const ASSETS_ROUTE_PREFIX = PROXY_ROUTE_PREFIX;
const CODER_AUTH_HEADER_KEY = "Coder-Session-Token";
const REQUEST_TIMEOUT_MS = 2e4;
function getCoderApiRequestInit(authToken) {
  return {
    headers: { [CODER_AUTH_HEADER_KEY]: authToken },
    signal: AbortSignal.timeout(REQUEST_TIMEOUT_MS)
  };
}
class BackstageHttpError extends Error {
  constructor(errorMessage, response) {
    super(errorMessage);
    __privateAdd(this, _response, void 0);
    this.name = "HttpError";
    __privateSet(this, _response, response);
  }
  get status() {
    return __privateGet(this, _response).status;
  }
  get ok() {
    return __privateGet(this, _response).ok;
  }
  get contentType() {
    return __privateGet(this, _response).headers.get("content_type");
  }
}
_response = new WeakMap();
async function getWorkspaces(fetchInputs) {
  var _a;
  const { baseUrl, coderQuery, auth } = fetchInputs;
  assertValidCoderAuth(auth);
  const urlParams = new URLSearchParams({
    q: coderQuery,
    limit: "0"
  });
  const response = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/workspaces?${urlParams.toString()}`,
    getCoderApiRequestInit(auth.token)
  );
  if (!response.ok) {
    throw new BackstageHttpError(
      `Unable to retrieve workspaces for query (${coderQuery})`,
      response
    );
  }
  if (!((_a = response.headers.get("content-type")) == null ? void 0 : _a.includes("application/json"))) {
    throw new BackstageHttpError(
      "200 request has no data - potential proxy issue",
      response
    );
  }
  const json = await response.json();
  const { workspaces: workspaces2 } = parse(workspacesResponseSchema, json);
  const withRemappedImgUrls = workspaces2.map((ws) => {
    const templateIcon = ws.template_icon;
    if (!templateIcon.startsWith("/")) {
      return ws;
    }
    return {
      ...ws,
      template_icon: `${baseUrl}${ASSETS_ROUTE_PREFIX}${templateIcon}`
    };
  });
  return withRemappedImgUrls;
}
async function getWorkspaceBuildParameters(inputs) {
  var _a;
  const { baseUrl, auth, workspaceBuildId } = inputs;
  assertValidCoderAuth(auth);
  const res = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/workspacebuilds/${workspaceBuildId}/parameters`,
    getCoderApiRequestInit(auth.token)
  );
  if (!res.ok) {
    throw new BackstageHttpError(
      `Failed to retreive build params for workspace ID ${workspaceBuildId}`,
      res
    );
  }
  if (!((_a = res.headers.get("content-type")) == null ? void 0 : _a.includes("application/json"))) {
    throw new BackstageHttpError(
      "200 request has no data - potential proxy issue",
      res
    );
  }
  const json = await res.json();
  return parse(workspaceBuildParametersSchema, json);
}
async function getWorkspacesByRepo(inputs) {
  const workspaces2 = await getWorkspaces(inputs);
  const paramResults = await Promise.allSettled(
    workspaces2.map(
      (ws) => getWorkspaceBuildParameters({
        ...inputs,
        workspaceBuildId: ws.latest_build.id
      })
    )
  );
  const { workspacesConfig } = inputs;
  const matchedWorkspaces = [];
  for (const [index, res] of paramResults.entries()) {
    if (res.status === "rejected") {
      continue;
    }
    for (const param of res.value) {
      const include = workspacesConfig.repoUrlParamKeys.includes(param.name) && param.value === workspacesConfig.repoUrl;
      if (include) {
        matchedWorkspaces.push(workspaces2[index]);
        break;
      }
    }
  }
  return matchedWorkspaces;
}
function getWorkspaceAgentStatuses(workspace) {
  const uniqueStatuses = [];
  for (const resource of workspace.latest_build.resources) {
    if (resource.agents === void 0) {
      continue;
    }
    for (const agent of resource.agents) {
      const status = agent.status;
      if (!uniqueStatuses.includes(status)) {
        uniqueStatuses.push(status);
      }
    }
  }
  return uniqueStatuses;
}
function workspaces(inputs) {
  const enabled = inputs.auth.status === "authenticated";
  return {
    queryKey: [CODER_QUERY_KEY_PREFIX, "workspaces", inputs.coderQuery],
    queryFn: () => getWorkspaces(inputs),
    enabled,
    keepPreviousData: enabled && inputs.coderQuery !== ""
  };
}
function workspacesByRepo(inputs) {
  const enabled = inputs.auth.status === "authenticated" && inputs.coderQuery !== "";
  return {
    queryKey: [CODER_QUERY_KEY_PREFIX, "workspaces", inputs.coderQuery, "repo"],
    queryFn: () => getWorkspacesByRepo(inputs),
    enabled,
    keepPreviousData: enabled
  };
}
async function isAuthValid$1(inputs) {
  const { baseUrl, authToken } = inputs;
  const response = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/users/me`,
    getCoderApiRequestInit(authToken)
  );
  if (response.status >= 400 && response.status !== 401) {
    throw new BackstageHttpError("Failed to complete request", response);
  }
  return response.status !== 401;
}
const authQueryKey = [CODER_QUERY_KEY_PREFIX, "auth"];
function authValidation(inputs) {
  const enabled = inputs.authToken !== "";
  return {
    queryKey: [...authQueryKey, inputs.authToken],
    queryFn: () => isAuthValid$1(inputs),
    enabled,
    keepPreviousData: enabled
  };
}

function useBackstageEndpoints() {
  const backstageConfig = useApi(configApiRef);
  const baseUrl = backstageConfig.getString("backend.baseUrl");
  return {
    baseUrl,
    assetsProxyUrl: `${baseUrl}${ASSETS_ROUTE_PREFIX}`,
    apiProxyUrl: `${baseUrl}${API_ROUTE_PREFIX}`
  };
}

const TOKEN_STORAGE_KEY = "coder-backstage-plugin/token";
const AUTH_GRACE_PERIOD_TIMEOUT_MS = 6e3;
function isAuthValid(state) {
  return state.status === "authenticated" || state.status === "distrustedWithGracePeriod";
}
function assertValidCoderAuth(auth) {
  if (!isAuthValid(auth)) {
    throw new Error("Coder auth is not valid");
  }
}
const AuthContext = createContext(null);
function useCoderAuth() {
  const contextValue = useContext(AuthContext);
  if (contextValue === null) {
    throw new Error(
      `Hook ${useCoderAuth.name} is being called outside of CoderProvider`
    );
  }
  return contextValue;
}
const CoderAuthProvider = ({ children }) => {
  const { baseUrl } = useBackstageEndpoints();
  const [isInsideGracePeriod, setIsInsideGracePeriod] = useState(true);
  const [authToken, setAuthToken] = useState(readAuthToken);
  const [readonlyInitialAuthToken] = useState(authToken);
  const authValidityQuery = useQuery({
    ...authValidation({ baseUrl, authToken }),
    refetchOnWindowFocus: (query) => query.state.data !== false
  });
  const authState = generateAuthState({
    authToken,
    authValidityQuery,
    isInsideGracePeriod,
    initialAuthToken: readonlyInitialAuthToken
  });
  if (!isInsideGracePeriod && authState.status === "authenticated") {
    setIsInsideGracePeriod(true);
  }
  useEffect(() => {
    if (authState.status === "authenticated") {
      window.localStorage.setItem(TOKEN_STORAGE_KEY, authState.token);
    }
  }, [authState.status, authState.token]);
  useEffect(() => {
    if (authState.status !== "distrustedWithGracePeriod") {
      return void 0;
    }
    const distrustTimeoutId = window.setTimeout(() => {
      setIsInsideGracePeriod(false);
    }, AUTH_GRACE_PERIOD_TIMEOUT_MS);
    return () => window.clearTimeout(distrustTimeoutId);
  }, [authState.status]);
  const queryClient = useQueryClient();
  useEffect(() => {
    let isRefetchingTokenQuery = false;
    const queryCache = queryClient.getQueryCache();
    const unsubscribe = queryCache.subscribe(async (event) => {
      const queryError = event.query.state.error;
      const shouldRevalidate = !isRefetchingTokenQuery && queryError instanceof BackstageHttpError && queryError.status === 401;
      if (!shouldRevalidate) {
        return;
      }
      isRefetchingTokenQuery = true;
      await queryClient.refetchQueries({ queryKey: authQueryKey });
      isRefetchingTokenQuery = false;
    });
    return unsubscribe;
  }, [queryClient]);
  return /* @__PURE__ */ React.createElement(
    AuthContext.Provider,
    {
      value: {
        ...authState,
        isAuthenticated: isAuthValid(authState),
        tokenLoadedOnMount: readonlyInitialAuthToken !== "",
        registerNewToken: (newToken) => {
          if (newToken !== "") {
            setAuthToken(newToken);
          }
        },
        ejectToken: () => {
          window.localStorage.removeItem(TOKEN_STORAGE_KEY);
          queryClient.removeQueries({ queryKey: [CODER_QUERY_KEY_PREFIX] });
          setAuthToken("");
        }
      }
    },
    children
  );
};
function generateAuthState({
  authToken,
  initialAuthToken,
  authValidityQuery,
  isInsideGracePeriod
}) {
  const isInitializing = initialAuthToken !== "" && authValidityQuery.isLoading && authValidityQuery.isFetching && !authValidityQuery.isFetchedAfterMount;
  if (isInitializing) {
    return {
      status: "initializing",
      token: void 0,
      error: void 0
    };
  }
  if (!authToken) {
    return {
      status: "tokenMissing",
      token: void 0,
      error: void 0
    };
  }
  if (authValidityQuery.error instanceof BackstageHttpError) {
    const deploymentLikelyUnavailable = authValidityQuery.error.status === 504 || authValidityQuery.error.status === 200 && authValidityQuery.error.contentType !== "application/json";
    if (deploymentLikelyUnavailable) {
      return {
        status: "deploymentUnavailable",
        token: void 0,
        error: authValidityQuery.error
      };
    }
  }
  const isTokenValidFromPrevFetch = authValidityQuery.data === true;
  if (isTokenValidFromPrevFetch) {
    const canTrustAuthThisRender = authValidityQuery.isSuccess && !authValidityQuery.isPaused;
    if (canTrustAuthThisRender) {
      return {
        status: "authenticated",
        token: authToken,
        error: void 0
      };
    }
    if (isInsideGracePeriod) {
      return {
        status: "distrustedWithGracePeriod",
        token: authToken,
        error: void 0
      };
    }
    return {
      status: "distrusted",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  const isAuthenticating = authValidityQuery.isLoading || authValidityQuery.isRefetching && (authValidityQuery.isError && authValidityQuery.data !== true || authValidityQuery.isSuccess && authValidityQuery.data === false);
  if (isAuthenticating) {
    return {
      status: "authenticating",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  const isCoderDeploymentDown = authValidityQuery.error instanceof Error && authValidityQuery.error.name === "TimeoutError";
  if (isCoderDeploymentDown) {
    return {
      status: "distrusted",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  if (authValidityQuery.isPaused) {
    return {
      status: "noInternetConnection",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  return {
    status: "invalid",
    token: void 0,
    error: authValidityQuery.error
  };
}
function readAuthToken() {
  var _a;
  return (_a = window.localStorage.getItem(TOKEN_STORAGE_KEY)) != null ? _a : "";
}

const AppConfigContext = createContext(null);
function useCoderAppConfig() {
  const value = useContext(AppConfigContext);
  if (value === null) {
    throw new Error(
      `Hook ${useCoderAppConfig.name} must be called from a CoderProvider component`
    );
  }
  return value;
}
const CoderAppConfigProvider = ({
  children,
  appConfig
}) => {
  return /* @__PURE__ */ React.createElement(AppConfigContext.Provider, { value: appConfig }, children);
};

const FallbackUi = () => {
  return /* @__PURE__ */ React.createElement("p", null, "Error encountered. Please check browser console.");
};
class ErrorBoundaryCore extends Component {
  static getDerivedStateFromError() {
    return { hasError: true };
  }
  constructor(props) {
    super(props);
    this.state = { hasError: false };
  }
  componentDidCatch(error, errorInfo) {
    this.props.onError(error, errorInfo.componentStack || "no details");
  }
  render() {
    const { children, fallbackUi } = this.props;
    return this.state.hasError ? fallbackUi : children;
  }
}
const CoderErrorBoundary$1 = ({
  children,
  fallbackUi
}) => {
  const errorApi = useApi(errorApiRef);
  const fallbackContent = fallbackUi != null ? fallbackUi : /* @__PURE__ */ React.createElement(FallbackUi, null);
  const onError = (error, componentStack) => {
    var _a;
    let errorToLog;
    if (error instanceof ValiError) {
      const logs = error.issues.map((issue) => issue.message).join(", ");
      errorToLog = new Error(logs);
    } else {
      errorToLog = new Error();
      errorToLog.message = error.message;
      errorToLog.name = error.name;
    }
    errorToLog.stack = `${(_a = error.stack) != null ? _a : ""}

React Component stack:
${componentStack}`;
    errorApi.post(errorToLog, {
      hidden: false
    });
  };
  return /* @__PURE__ */ React.createElement(ErrorBoundaryCore, { fallbackUi: fallbackContent, onError }, children);
};

const MAX_FETCH_FAILURES = 3;
const shouldRetryRequest = (failureCount, error) => {
  const isBelowThreshold = failureCount < MAX_FETCH_FAILURES;
  if (!(error instanceof BackstageHttpError)) {
    return isBelowThreshold;
  }
  const isAuthenticationError = error.status === 401;
  const isLikelyProxyConfigurationError = error.status === 504 || error.status === 200 && error.contentType !== "application/json";
  return !isAuthenticationError && !isLikelyProxyConfigurationError && isBelowThreshold;
};
const defaultClient = new QueryClient({
  defaultOptions: {
    queries: {
      retry: shouldRetryRequest
    },
    mutations: {
      retry: shouldRetryRequest
    }
  }
});
const CoderProvider$1 = ({
  children,
  appConfig,
  queryClient = defaultClient
}) => {
  return /* @__PURE__ */ React.createElement(CoderErrorBoundary$1, null, /* @__PURE__ */ React.createElement(QueryClientProvider, { client: queryClient }, /* @__PURE__ */ React.createElement(CoderAppConfigProvider, { appConfig }, /* @__PURE__ */ React.createElement(CoderAuthProvider, null, children))));
};

const workspaceCreationModeSchema = optional(
  union(
    [literal("manual"), literal("auto")],
    "If defined, createMode must be 'manual' or 'auto'"
  )
);
const yamlConfigSchema = union([
  undefined_(),
  object({
    templateName: optional(string()),
    mode: workspaceCreationModeSchema,
    params: optional(
      record(
        string(),
        // Defining record value with undefined case as a safety net if user
        // hasn't or can't turn on the noUncheckedIndexedAccess compiler option
        union([string(), undefined_()]),
        "If defined, params must be JSON-serializable as Record<string, string>"
      )
    )
  })
]);
function compileCoderConfig(appConfig, rawYamlConfig, repoUrl) {
  var _a, _b, _c, _d;
  const { workspaces, deployment } = appConfig;
  const yamlConfig = parse(yamlConfigSchema, rawYamlConfig);
  const mode = (_b = (_a = yamlConfig == null ? void 0 : yamlConfig.mode) != null ? _a : workspaces.defaultMode) != null ? _b : "manual";
  const templateName = (_c = yamlConfig == null ? void 0 : yamlConfig.templateName) != null ? _c : workspaces.defaultTemplateName;
  const urlParams = new URLSearchParams({ mode });
  const compiledParams = {};
  const paramsPrecedence = [workspaces.params, (_d = yamlConfig == null ? void 0 : yamlConfig.params) != null ? _d : {}];
  for (const params of paramsPrecedence) {
    for (const key in params) {
      if (!params.hasOwnProperty(key)) {
        continue;
      }
      const value = params[key];
      if (typeof value === "string") {
        compiledParams[key] = value;
        urlParams.set(`param.${key}`, value);
      }
    }
  }
  let cleanedRepoUrl = repoUrl;
  if (repoUrl !== void 0) {
    cleanedRepoUrl = repoUrl.replace(/\/tree\/[\w._-]+\/?$/, "");
    for (const key of workspaces.repoUrlParamKeys) {
      compiledParams[key] = cleanedRepoUrl;
      urlParams.set(`param.${key}`, cleanedRepoUrl);
    }
  }
  let creationUrl = void 0;
  if (templateName) {
    const safeTemplate = encodeURIComponent(templateName);
    creationUrl = `${deployment.accessUrl}/templates/${safeTemplate}/workspace?${urlParams.toString()}`;
  }
  return {
    mode,
    creationUrl,
    templateName,
    repoUrl: cleanedRepoUrl,
    isReadingEntityData: yamlConfig !== void 0,
    repoUrlParamKeys: workspaces.repoUrlParamKeys,
    params: compiledParams
  };
}
function useCoderWorkspacesConfig({
  readEntityData = false
}) {
  const appConfig = useCoderAppConfig();
  const { rawYaml, repoUrl } = useDynamicEntity(readEntityData);
  return useMemo(
    () => compileCoderConfig(appConfig, rawYaml, repoUrl),
    // Backstage seems to have stabilized the value of rawYamlConfig, so even
    // when it's an object, useMemo shouldn't re-run unnecessarily
    [appConfig, rawYaml, repoUrl]
  );
}
function useDynamicEntity(readEntityData) {
  var _a;
  const [initialReadSetting] = useState(readEntityData);
  if (readEntityData !== initialReadSetting) {
    throw new Error(
      'The value of "readEntityData" is not allowed to change across re-renders'
    );
  }
  let rawYaml = void 0;
  let repoUrl = void 0;
  if (readEntityData) {
    const { entity } = useEntity();
    const sourceControlApi = useApi(scmIntegrationsApiRef);
    const repoData = getEntitySourceLocation(entity, sourceControlApi);
    rawYaml = (_a = entity.spec) == null ? void 0 : _a.coder;
    repoUrl = repoData == null ? void 0 : repoData.locationTargetUrl;
  }
  return { rawYaml, repoUrl };
}

function useCoderWorkspacesQuery({
  coderQuery,
  workspacesConfig
}) {
  const auth = useCoderAuth();
  const { baseUrl } = useBackstageEndpoints();
  const hasRepoData = workspacesConfig && workspacesConfig.repoUrl;
  const queryOptions = hasRepoData ? workspacesByRepo({ coderQuery, auth, baseUrl, workspacesConfig }) : workspaces({ coderQuery, auth, baseUrl });
  return useQuery(queryOptions);
}

let idCounter = 0;
function useIdPolyfill() {
  const [readonlyId] = useState(() => {
    idCounter++;
    return `:r${idCounter}:`;
  });
  return readonlyId;
}
const useId = typeof React.useId === "undefined" ? useIdPolyfill : React.useId;

const useStyles$5 = makeStyles((theme) => ({
  root: {
    color: theme.palette.type,
    backgroundColor: theme.palette.background.paper,
    padding: theme.spacing(2),
    borderRadius: theme.shape.borderRadius,
    boxShadow: theme.shadows[1]
  }
}));
const Card = forwardRef((props, ref) => {
  const { className, ...delegatedProps } = props;
  const styles = useStyles$5();
  return /* @__PURE__ */ React.createElement(
    "div",
    {
      ref,
      className: `${styles.root} ${className != null ? className : ""}`,
      ...delegatedProps
    }
  );
});

const useStyles$4 = makeStyles((theme) => ({
  root: {
    fill: theme.palette.text.primary,
    opacity: 1
  }
}));
const CoderLogo = ({
  className = "",
  ...delegatedProps
}) => {
  const styles = useStyles$4();
  return /* @__PURE__ */ React.createElement(
    "svg",
    {
      "aria-hidden": true,
      className: `${styles.root} ${className}`,
      viewBox: "0 0 68 49",
      width: "40",
      ...delegatedProps
    },
    /* @__PURE__ */ React.createElement("path", { d: "M66.3575 21.3584C65.0024 21.3584 64.099 20.5638 64.099 18.9328V9.5647C64.099 3.58419 61.6353 0.280273 55.2705 0.280273H52.314V6.59536H53.2174C55.7222 6.59536 56.913 7.97547 56.913 10.443V18.7237C56.913 22.3203 57.9807 23.7841 60.3212 24.5369C57.9807 25.2479 56.913 26.7534 56.913 30.3501C56.913 32.3994 56.913 34.4486 56.913 36.4979C56.913 38.2126 56.913 39.8855 56.4613 41.6002C56.0097 43.1894 55.2705 44.695 54.244 45.9914C53.6691 46.7442 53.0121 47.3716 52.2729 47.9571V48.7935H55.2295C61.5942 48.7935 64.058 45.4896 64.058 39.5091V30.141C64.058 28.4681 64.9203 27.7153 66.3164 27.7153H68V21.4003H66.3575V21.3584Z" }),
    /* @__PURE__ */ React.createElement("path", { d: "M46.2367 9.81532H37.1208C36.9155 9.81532 36.7512 9.64804 36.7512 9.43893V8.72796C36.7512 8.51885 36.9155 8.35156 37.1208 8.35156H46.2778C46.4831 8.35156 46.6473 8.51885 46.6473 8.72796V9.43893C46.6473 9.64804 46.442 9.81532 46.2367 9.81532Z" }),
    /* @__PURE__ */ React.createElement("path", { d: "M47.7971 18.8485H41.145C40.9396 18.8485 40.7754 18.6812 40.7754 18.4721V17.7612C40.7754 17.5521 40.9396 17.3848 41.145 17.3848H47.7971C48.0024 17.3848 48.1667 17.5521 48.1667 17.7612V18.4721C48.1667 18.6394 48.0024 18.8485 47.7971 18.8485Z" }),
    /* @__PURE__ */ React.createElement("path", { d: "M50.4251 14.3319H37.1208C36.9155 14.3319 36.7512 14.1646 36.7512 13.9555V13.2446C36.7512 13.0355 36.9155 12.8682 37.1208 12.8682H50.384C50.5894 12.8682 50.7536 13.0355 50.7536 13.2446V13.9555C50.7536 14.1228 50.6304 14.3319 50.4251 14.3319Z" }),
    /* @__PURE__ */ React.createElement("path", { d: "M26.5677 11.8649C27.4711 11.8649 28.3744 11.9485 29.2368 12.1577V10.443C29.2368 8.0173 30.4686 6.59536 32.9324 6.59536H33.8358V0.280273H30.8793C24.5145 0.280273 22.0508 3.58419 22.0508 9.5647V12.6595C23.488 12.1577 25.0073 11.8649 26.5677 11.8649Z" }),
    /* @__PURE__ */ React.createElement("path", { d: "M53.2174 34.6165C52.5603 29.3051 48.5362 24.872 43.3623 23.8683C41.9251 23.5755 40.4879 23.5337 39.0918 23.7847C39.0507 23.7847 39.0507 23.7428 39.0096 23.7428C36.7512 18.9333 31.9058 15.7549 26.6497 15.7549C21.3937 15.7549 16.5894 18.8497 14.2898 23.6592C14.2488 23.6592 14.2488 23.701 14.2077 23.701C12.7295 23.5337 11.2512 23.6174 9.77294 23.9938C4.68116 25.2484 0.821255 29.5979 0.123188 34.8674C0.0410628 35.4111 0 35.9548 0 36.4566C0 38.0459 1.06763 39.5096 2.62802 39.7187C4.55797 40.0115 6.24154 38.5059 6.20048 36.5821C6.20048 36.2894 6.20048 35.9548 6.24154 35.662C6.57004 32.9854 8.58212 30.7271 11.2101 30.0997C12.0314 29.8906 12.8526 29.8488 13.6328 29.9743C16.1377 30.3088 18.6014 29.0124 19.6691 26.754C20.4493 25.0811 21.6811 23.6174 23.3237 22.8228C25.1304 21.9445 27.1836 21.819 29.0724 22.4882C31.0435 23.1992 32.5217 24.7047 33.4251 26.5867C34.3695 28.4269 34.8212 29.7233 36.8333 29.9743C37.6546 30.0997 39.9541 30.0579 40.8164 30.0161C42.5 30.0161 44.1835 30.6016 45.3744 31.8144C46.1546 32.6509 46.7294 33.6964 46.9758 34.8674C47.3454 36.7494 46.8937 38.6314 45.785 40.0533C45.0048 41.057 43.9372 41.8098 42.7463 42.1444C42.1715 42.3117 41.5966 42.3535 41.0217 42.3535C40.6932 42.3535 40.2415 42.3535 39.7077 42.3535C38.0652 42.3535 34.5749 42.3535 31.9468 42.3535C30.1401 42.3535 28.7029 40.8897 28.7029 39.0496V32.86V26.7958C28.7029 26.294 28.2922 25.8757 27.7995 25.8757H26.5265C24.0217 25.9176 22.0096 28.7614 22.0096 31.7726C22.0096 34.7838 22.0096 42.7717 22.0096 42.7717C22.0096 46.0338 24.5966 48.6686 27.7995 48.6686C27.7995 48.6686 42.0483 48.6268 42.2536 48.6268C45.5386 48.2922 48.5773 46.5775 50.6304 43.9427C52.6835 41.3916 53.628 38.0459 53.2174 34.6165Z" })
  );
};

const useStyles$3 = makeStyles((theme) => ({
  root: {
    display: "flex",
    flexFlow: "column nowrap",
    alignItems: "center",
    maxWidth: "30em",
    marginLeft: "auto",
    marginRight: "auto",
    rowGap: theme.spacing(2)
  },
  button: {
    maxWidth: "fit-content",
    marginLeft: "auto",
    marginRight: "auto"
  },
  coderLogo: {
    display: "block",
    width: "fit-content",
    marginLeft: "auto",
    marginRight: "auto"
  }
}));
const CoderAuthDistrustedForm = () => {
  const styles = useStyles$3();
  const { ejectToken } = useCoderAuth();
  return /* @__PURE__ */ React.createElement("div", { className: styles.root }, /* @__PURE__ */ React.createElement("div", null, /* @__PURE__ */ React.createElement(CoderLogo, { className: styles.coderLogo }), /* @__PURE__ */ React.createElement("p", null, "Unable to verify token authenticity. Please check your internet connection, or try ejecting the token.")), /* @__PURE__ */ React.createElement(
    LinkButton,
    {
      disableRipple: true,
      to: "",
      component: "button",
      type: "submit",
      color: "primary",
      variant: "contained",
      className: styles.button,
      onClick: ejectToken
    },
    "Eject token"
  ));
};

const visuallyHiddenStyles = {
  display: "inline-block",
  position: "absolute",
  overflow: "hidden",
  clip: "rect(0 0 0 0)",
  height: 1,
  width: 1,
  margin: -1,
  padding: 0,
  border: 0
};
const VisuallyHidden = ({
  children,
  ...delegatedProps
}) => {
  const [forceShow, setForceShow] = useState(false);
  useEffect(() => {
    if (process.env.NODE_ENV === "production") {
      return void 0;
    }
    const handleKeyDown = (event) => {
      if (event.shiftKey && event.key === "Alt") {
        setForceShow(true);
      }
    };
    const handleKeyUp = (event) => {
      if (event.key === "Alt") {
        setForceShow(false);
      }
    };
    window.addEventListener("keydown", handleKeyDown);
    window.addEventListener("keyup", handleKeyUp);
    return () => {
      window.removeEventListener("keydown", handleKeyDown);
      window.removeEventListener("keyup", handleKeyUp);
    };
  }, []);
  if (forceShow) {
    return /* @__PURE__ */ React.createElement(React.Fragment, null, children);
  }
  return /* @__PURE__ */ React.createElement("span", { style: visuallyHiddenStyles, ...delegatedProps }, children);
};

const MAX_DOTS = 3;
const dotRange = new Array(MAX_DOTS).fill(null).map((_, i) => i + 1);
const useStyles$2 = makeStyles((theme) => ({
  root: {
    display: "flex",
    flexFlow: "column nowrap",
    alignItems: "center"
  },
  text: {
    lineHeight: theme.typography.body1.lineHeight,
    paddingLeft: theme.spacing(1)
  },
  coderLogo: {
    display: "block",
    width: "fit-content",
    marginLeft: "auto",
    marginRight: "auto"
  }
}));
const CoderAuthLoadingState = () => {
  const [visibleDots, setVisibleDots] = useState(0);
  const styles = useStyles$2();
  useEffect(() => {
    const intervalId = window.setInterval(() => {
      setVisibleDots((current) => (current + 1) % (MAX_DOTS + 1));
    }, 1e3);
    return () => window.clearInterval(intervalId);
  }, []);
  return /* @__PURE__ */ React.createElement("div", { className: styles.root }, /* @__PURE__ */ React.createElement(CoderLogo, { className: styles.coderLogo }), /* @__PURE__ */ React.createElement("p", { className: styles.text }, "Loading", /* @__PURE__ */ React.createElement(VisuallyHidden, null, "\u2026"), dotRange.map((dotPosition) => /* @__PURE__ */ React.createElement(
    "span",
    {
      key: dotPosition,
      style: { opacity: visibleDots >= dotPosition ? 1 : 0 },
      "aria-hidden": true
    },
    "."
  ))));
};

const useStyles$1 = makeStyles((theme) => ({
  formContainer: {
    maxWidth: "30em",
    marginLeft: "auto",
    marginRight: "auto"
  },
  authInputFieldset: {
    display: "flex",
    flexFlow: "column nowrap",
    rowGap: theme.spacing(2),
    margin: `${theme.spacing(-0.5)} 0 0 0`,
    border: "none",
    padding: 0
  },
  coderLogo: {
    display: "block",
    width: "fit-content",
    marginLeft: "auto",
    marginRight: "auto"
  },
  authButton: {
    display: "block",
    width: "fit-content",
    marginLeft: "auto",
    marginRight: "auto"
  }
}));
const CoderAuthInputForm = () => {
  const hookId = useId();
  const styles = useStyles$1();
  const appConfig = useCoderAppConfig();
  const { status, registerNewToken } = useCoderAuth();
  const onSubmit = (event) => {
    event.preventDefault();
    const formData = Object.fromEntries(new FormData(event.currentTarget));
    const newToken = typeof formData.authToken === "string" ? formData.authToken : "";
    registerNewToken(newToken);
  };
  const formHeaderId = `${hookId}-form-header`;
  const legendId = `${hookId}-legend`;
  const authTokenInputId = `${hookId}-auth-token`;
  const warningBannerId = `${hookId}-warning-banner`;
  return /* @__PURE__ */ React.createElement(
    "form",
    {
      "aria-labelledby": formHeaderId,
      className: styles.formContainer,
      onSubmit
    },
    /* @__PURE__ */ React.createElement("h3", { hidden: true, id: formHeaderId }, "Authenticate with Coder"),
    /* @__PURE__ */ React.createElement("div", null, /* @__PURE__ */ React.createElement(CoderLogo, { className: styles.coderLogo }), /* @__PURE__ */ React.createElement("p", null, "Link your Coder account to create remote workspaces. Please enter a new token from your", " ", /* @__PURE__ */ React.createElement(
      Link,
      {
        to: `${appConfig.deployment.accessUrl}/cli-auth`,
        target: "_blank"
      },
      "Coder deployment's token page",
      /* @__PURE__ */ React.createElement(VisuallyHidden, null, " (link opens in new tab)")
    ), ".")),
    /* @__PURE__ */ React.createElement("fieldset", { className: styles.authInputFieldset, "aria-labelledby": legendId }, /* @__PURE__ */ React.createElement("legend", { hidden: true, id: legendId }, "Auth input"), /* @__PURE__ */ React.createElement(
      TextField,
      {
        label: "Auth token",
        InputLabelProps: { htmlFor: authTokenInputId },
        InputProps: { id: authTokenInputId },
        required: true,
        name: "authToken",
        type: "password",
        defaultValue: "",
        "aria-errormessage": warningBannerId,
        style: { width: "100%" }
      }
    ), /* @__PURE__ */ React.createElement(
      LinkButton,
      {
        disableRipple: true,
        to: "",
        component: "button",
        type: "submit",
        color: "primary",
        variant: "contained",
        className: styles.authButton
      },
      "Authenticate"
    )),
    (status === "invalid" || status === "authenticating") && /* @__PURE__ */ React.createElement(InvalidStatusNotifier, { authStatus: status, bannerId: warningBannerId })
  );
};
const useInvalidStatusStyles = makeStyles((theme) => ({
  warningBannerSpacer: {
    paddingTop: theme.spacing(2)
  },
  warningBanner: {
    display: "flex",
    flexFlow: "row nowrap",
    alignItems: "center",
    color: theme.palette.text.primary,
    backgroundColor: theme.palette.background.default,
    borderRadius: theme.shape.borderRadius,
    border: `1.5px solid ${theme.palette.background.default}`,
    padding: 0
  },
  errorContent: {
    display: "flex",
    flexFlow: "row nowrap",
    alignItems: "center",
    columnGap: theme.spacing(1),
    marginRight: "auto",
    paddingTop: theme.spacing(0.5),
    paddingBottom: theme.spacing(0.5),
    paddingLeft: theme.spacing(2),
    paddingRight: 0
  },
  icon: {
    fontSize: "16px"
  },
  syncIcon: {
    color: theme.palette.text.primary,
    opacity: 0.6
  },
  errorIcon: {
    color: theme.palette.error.main,
    fontSize: "16px"
  },
  dismissButton: {
    border: "none",
    alignSelf: "stretch",
    padding: `0 ${theme.spacing(1.5)}px 0 ${theme.spacing(2)}px`,
    color: theme.palette.text.primary,
    backgroundColor: "inherit",
    lineHeight: 1,
    cursor: "pointer",
    "&:hover": {
      backgroundColor: theme.palette.action.hover
    }
  },
  "@keyframes spin": {
    "100%": {
      transform: "rotate(360deg)"
    }
  }
}));
function InvalidStatusNotifier({ authStatus, bannerId }) {
  const [showNotification, setShowNotification] = useState(true);
  const styles = useInvalidStatusStyles();
  if (!showNotification) {
    return null;
  }
  return /* @__PURE__ */ React.createElement("div", { className: styles.warningBannerSpacer }, /* @__PURE__ */ React.createElement("div", { id: bannerId, className: styles.warningBanner }, /* @__PURE__ */ React.createElement("span", { className: styles.errorContent }, authStatus === "authenticating" && /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(
    SyncIcon,
    {
      className: `${styles.icon} ${styles.syncIcon}`,
      fontSize: "inherit"
    }
  ), "Authenticating\u2026"), authStatus === "invalid" && /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(
    ErrorIcon,
    {
      className: `${styles.icon} ${styles.errorIcon}`,
      fontSize: "inherit"
    }
  ), "Invalid token")), /* @__PURE__ */ React.createElement(
    "button",
    {
      className: styles.dismissButton,
      onClick: () => setShowNotification(false)
    },
    "Dismiss"
  )));
}

const useStyles = makeStyles((theme) => ({
  cardContent: {
    paddingTop: theme.spacing(5),
    paddingBottom: theme.spacing(5)
  }
}));
function CoderAuthCard({ children }) {
  const styles = useStyles();
  return /* @__PURE__ */ React.createElement(InfoCard, { title: "Authenticate with Coder" }, /* @__PURE__ */ React.createElement("div", { className: styles.cardContent }, children));
}
const CoderAuthWrapper$1 = ({ children, type }) => {
  const auth = useCoderAuth();
  if (auth.isAuthenticated) {
    return /* @__PURE__ */ React.createElement(React.Fragment, null, children);
  }
  let Wrapper;
  switch (type) {
    case "card": {
      Wrapper = CoderAuthCard;
      break;
    }
    default: {
      assertExhaustion(type);
    }
  }
  return /* @__PURE__ */ React.createElement(Wrapper, null, (() => {
    switch (auth.status) {
      case "initializing": {
        return /* @__PURE__ */ React.createElement(CoderAuthLoadingState, null);
      }
      case "distrusted":
      case "noInternetConnection":
      case "deploymentUnavailable": {
        return /* @__PURE__ */ React.createElement(CoderAuthDistrustedForm, null);
      }
      case "authenticating":
      case "invalid":
      case "tokenMissing": {
        return /* @__PURE__ */ React.createElement(CoderAuthInputForm, null);
      }
      case "authenticated":
      case "distrustedWithGracePeriod": {
        throw new Error(
          "Tried to process authenticated user after main content should already be shown"
        );
      }
      default: {
        return assertExhaustion(auth);
      }
    }
  })());
};
function assertExhaustion(...inputs) {
  let inputsToLog;
  try {
    inputsToLog = JSON.stringify(inputs);
  } catch {
    inputsToLog = inputs;
  }
  throw new Error(
    `Not all possibilities for inputs (${inputsToLog}) have been exhausted`
  );
}

const CardContext = createContext(null);
const InnerRoot = ({
  children,
  className,
  queryFilter: outerFilter,
  onFilterChange: onOuterFilterChange,
  defaultQueryFilter = "owner:me",
  readEntityData = false,
  ...delegatedProps
}) => {
  const [innerFilter, setInnerFilter] = useState(defaultQueryFilter);
  const activeFilter = outerFilter != null ? outerFilter : innerFilter;
  const workspacesConfig = useCoderWorkspacesConfig({ readEntityData });
  const workspacesQuery = useCoderWorkspacesQuery({
    workspacesConfig,
    coderQuery: activeFilter
  });
  const hookId = useId();
  const headerId = `${hookId}-header`;
  return /* @__PURE__ */ React.createElement(CoderAuthWrapper$1, { type: "card" }, /* @__PURE__ */ React.createElement(
    CardContext.Provider,
    {
      value: {
        headerId,
        workspacesQuery,
        workspacesConfig,
        queryFilter: activeFilter,
        onFilterChange: (newFilter) => {
          setInnerFilter(newFilter);
          onOuterFilterChange == null ? void 0 : onOuterFilterChange(newFilter);
        }
      }
    },
    /* @__PURE__ */ React.createElement(Card, { role: "search", "aria-labelledby": headerId, ...delegatedProps }, /* @__PURE__ */ React.createElement("div", { role: "form" }, children))
  ));
};
function Root(props) {
  var _a;
  const renderKey = String((_a = props.readEntityData) != null ? _a : false);
  return /* @__PURE__ */ React.createElement(InnerRoot, { key: renderKey, ...props });
}
function useWorkspacesCardContext() {
  const contextValue = useContext(CardContext);
  if (contextValue === null) {
    throw new Error(
      `Not calling ${useWorkspacesCardContext.name} from inside a ${Root.name}`
    );
  }
  return contextValue;
}

const coderPlugin = createPlugin({
  id: "coder",
  routes: {
    root: rootRouteRef
  }
});
const CoderProvider = coderPlugin.provide(
  createComponentExtension({
    name: "CoderProvider",
    component: {
      lazy: () => import('./index-96dc1cb0.esm.js').then((m) => m.CoderProvider)
    }
  })
);
const CoderAuthWrapper = coderPlugin.provide(
  createComponentExtension({
    name: "CoderAuthWrapper",
    component: {
      lazy: () => import('./index-5c425620.esm.js').then((m) => m.CoderAuthWrapper)
    }
  })
);
const CoderErrorBoundary = coderPlugin.provide(
  createComponentExtension({
    name: "CoderErrorBoundary",
    component: {
      lazy: () => import('./index-265b8701.esm.js').then(
        (m) => m.CoderErrorBoundary
      )
    }
  })
);
const CoderWorkspacesCard = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.CoderWorkspacesCard
      )
    }
  })
);
const CoderWorkspacesCardCreateWorkspacesLink = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.CreateWorkspacesLink",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.CreateWorkspaceLink
      )
    }
  })
);
const CoderWorkspacesCardExtraActionsButton = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.ExtraActionsButton",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.ExtraActionsButton
      )
    }
  })
);
const CoderWorkspacesCardHeaderRow = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.HeaderRow",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then((m) => m.HeaderRow)
    }
  })
);
const CoderWorkspacesCardRoot = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.Root",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then((m) => m.Root)
    }
  })
);
const CoderWorkspacesCardSearchBox = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.SearchBox",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then((m) => m.SearchBox)
    }
  })
);
const CoderWorkspacesCardWorkspacesList = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.WorkspacesList",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then((m) => m.WorkspacesList)
    }
  })
);
const CoderWorkspacesCardWorkspacesListIcon = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.WorkspacesListIcon",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.WorkspacesListIcon
      )
    }
  })
);
const CoderWorkspacesCardWorkspacesListItem = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.WorkspacesListItem",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.WorkspacesListItem
      )
    }
  })
);
const CoderWorkspacesReminderAccordion = coderPlugin.provide(
  createComponentExtension({
    name: "CoderWorkspacesCard.ReminderAccordion",
    component: {
      lazy: () => import('./index-4e3e1dab.esm.js').then(
        (m) => m.ReminderAccordion
      )
    }
  })
);

export { AuthContext as A, CoderWorkspacesReminderAccordion as B, CoderAuthProvider as C, useCoderWorkspacesConfig as D, useCoderWorkspacesQuery as E, Root as R, VisuallyHidden as V, assertValidCoderAuth as a, useCoderAppConfig as b, CoderAppConfigProvider as c, CoderProvider$1 as d, CoderAuthWrapper$1 as e, CoderErrorBoundary$1 as f, useWorkspacesCardContext as g, useId as h, useBackstageEndpoints as i, getWorkspaceAgentStatuses as j, CoderLogo as k, CardContext as l, coderPlugin as m, CoderProvider as n, CoderAuthWrapper as o, CoderErrorBoundary as p, CoderWorkspacesCard as q, CoderWorkspacesCardCreateWorkspacesLink as r, CoderWorkspacesCardExtraActionsButton as s, CoderWorkspacesCardHeaderRow as t, useCoderAuth as u, CoderWorkspacesCardRoot as v, CoderWorkspacesCardSearchBox as w, CoderWorkspacesCardWorkspacesList as x, CoderWorkspacesCardWorkspacesListIcon as y, CoderWorkspacesCardWorkspacesListItem as z };
//# sourceMappingURL=index-553c43ff.esm.js.map
